/** FORMAT FOR STATUS LIGHT/TEXT CONTAINER  **/
.test-general-settings{
    width:50px;
    height:25px;
    margin:1px auto;
}

/** POSITION OF STATUS TEXT **/
.test-text {
    margin: 1px auto;
    position:relative;
    top: 5px;
    width: 40px;
    height: 40px;
}

/** FAIL STATUS LIGHT **/
.test-fail-msg {
    margin: 1px auto;
    position:relative;
    bottom:50px;
    right:60px;
    width: 40px;
    height: 40px;
    padding-left: 2px;
    border-radius: 50%;
    color: #da4e49;
}

.test-fail-msg:hover{
    color: #D47875;
}

/** PENDING STATUS LIGHT **/
.test-pending-msg {
    margin: 1px auto;
    position:relative;
    bottom:50px;
    right:55px;
    width: 40px;
    height: 40px;
    padding-left: 2px;
    color: #C9B501;
    border-radius: 50%;
}
.test-pending-msg:hover{
    color: #D8D085;
}

/** PASS STATUS LIGHT**/
.test-pass-msg {
    margin: 1px auto;
    position:relative;
    bottom:50px;
    right:60px;
    width: 40px;
    height: 40px;
    padding-left: 2px;
    color: #690;
    border-radius: 50%;
}
.test-pass-msg:hover{
    color: #A9CA65;
}

