.big-switch-logo {
	width: 206px;
	height: 50px;
	margin: 10px 10px 0px 0px;
}

.page-header-container .page-header-text{
	position: absolute;
	top: 25%;
}

.page-header-container .page-header h2 {
	position: relative;
}

