{% load i18n sizeformat %}

<h3>{% trans "Test Overview" %}</h3>

<div class="info row-fluid detail">
  <h4>{% trans "Info" %}</h4>
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Name" %}</dt>
    <dd>{{ reachabilitytest.name|default:_("None") }}</dd>
    <dt>{% trans "Connection Source" %}</dt>
    <dd>Source Tenant: {{ reachabilitytest.src_tenant_name|default:_("None") }}</dd>
    <dd>Source Segment: {{ reachabilitytest.src_segment_name|default:_("None") }}</dd>
    <dd>Source IP: {{ reachabilitytest.src_ip|default:_("None") }}</dd>
    <dt>{% trans "Connection Destination" %}</dt>
    <dd>Destination IP: {{ reachabilitytest.dst_ip|default:_("None") }}</dd>
  </dl>
</div>

<div class="specs row-fluid detail">
  <h4>{% trans "Results" %}</h4>
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Time Stamp" %}</dt>
    <dd>{{ reachabilitytest.test_time|default:_("None") }}</dd>
    <dt>{% trans "Expected Result" %}</dt>
    <dd>{{ reachabilitytest.expected_result|default:_("None") }}</dd>
    <dt>{% trans "Test Results" %}</dt>
    <dd>{{ reachabilitytest.test_result|default:_("None") }}</dd>
    <dt>{% trans "Command Line Output" %}</dt>
    <div class="command-prompt-background">
    	<ul class="command-prompt-text"></ul>
    </div>
  </dl>
</div>

